<?php
require_once('library.php');
require_once('message.php');
require_once('message_summary.php');
class SimpleMailClient
{
	private $mbox_str;
	
	private $username;
	private $password;
	
	private $mbox;
	
	public $connected = false;
	public $inbox = 'INBOX';
	
	public $current_mailbox;
	
	private $boxes;
	
	/**
	*	@type object
	*	@properties
	*		-Date current system time formatted according to RFC2822
	*		-Driver protocol used to access this mailbox: POP3, IMAP, NNTP
	*		-Mailbox the mailbox name
	*		-Nmsgs number of messages in the mailbox
	*		-Recent number of recent messages in the mailbox
	*		-Unseen number of unread messages in the mailbox
	*/
	public $status;
	
	public function __construct($host, $username, $password, $flags)
	{
		$flags = (count($flags) > 0) ? implode('/', $flags) : '';
		
		$this->mbox_str = '{' . $host . '/' . $flags . '}';
		$this->username = $username;
		$this->password = $password;
	}
	
	public function __destruct()
	{
		if($this->mbox)
			imap_close($this->mbox, CL_EXPUNGE);
	}
	
	/**
	*	Connect to the specified mailbox. Also populates the $status var
	*
	*	@param $box (string) The mailbox to connect to
	*	@return (boolean) True if the connection was successful, false otherwise
	*/
	public function connect($box = '')
	{
		if($this->connected)
			imap_close($this->mbox);
			
		$mbox = imap_open($this->mbox_str . $box, $this->username, $this->password);
		
		if($mbox)
		{
			$this->mbox = $mbox;
			$this->current_mailbox = $box;
			
			//set status
			$this->status = (object) imap_check($this->mbox);
			$stat = imap_status($this->mbox, $this->mbox_str . $box, SA_UNSEEN);

			$this->status->unseen = $stat->unseen;
			
			$this->connected = true;
			
			return true;
		}
		$this->connected = false;
		return false;
	}
	
	/**
	*	Opens a connection but doesn't connect to a mailbox (ie OP_HALFOPEN)
	*	
	*	@return (boolean) True on success, false otherwise
	*/
	public function quickConnect()
	{
		//if we already have a connection we don't need to open a new one
		if($this->connected) 
			return true;
		
		$mbox = imap_open($this->mbox_str, $this->username, $this->password, OP_HALFOPEN);
		
		if(SM_DEBUG && !$mbox)
		{
			print_r(imap_errors());
			print_r($this);
		}
		
		if($mbox)
		{
			$this->mbox = $mbox;
			$this->connected = true;
			
			$stat = imap_status($this->mbox, $this->mbox_str . $this->inbox, SA_UNSEEN);
			$this->status->unseen = $stat->unseen;
			
			return true;
		}
		return false;
	}
	
	/**
	*	Retrieves a sequential list of messages. Messages are paginated.
	*	Calling messageList(1, 5) will return the first 5 messages. Calling
	*	messageList(2, 5) will return the next 5 messages.
	*	
	*	@param $page (int) The page number, starting from one.
	*	@param $count (int) The size of the page.
	*
	*	@return (array) An array of SimpleMailMessageSummary objects. See
	*/
	public function messageList($page, $count = 0)
	{
		if($page == 0)
		{
			$start = $this->status->Nmsgs;
			$end = 1;
		}
		else
		{
			$start = $this->status->Nmsgs - (($page - 1) * $count);
			$end = $start - $count + 1;
			
			if($this->status->Nmsgs <= $count)
			{
				$start = $this->status->Nmsgs;
				$end = 1;
			}
			elseif($end < 1)
				$end = 1;
		}
		
		$overviews = imap_fetch_overview($this->mbox, "$start:$end");
		
		if(!$overviews)
			return array();
		
		foreach($overviews as &$o)
		{
			$o->from 	= @iconv_mime_decode($o->from);
			$o->to 		= @iconv_mime_decode($o->to);
			$o->date 	= @iconv_mime_decode($o->date);
			$o->subject = @iconv_mime_decode($o->subject);
		}

		return array_reverse($overviews);
	}
	
	public function search($query, $criteria)
	{
		$messages = imap_search($this->mbox, $criteria . " \"$query\"", SE_UID);
		if(!is_array($messages) || count($messages) == 0)
			return array();
		
		$msg_str = implode(',', $messages);
		
		$overviews = imap_fetch_overview($this->mbox, $msg_str, FT_UID);
		
		if(!$overviews)
			return array();
		
		foreach($overviews as &$o)
		{
			$o->from 	= @iconv_mime_decode($o->from);
			$o->to 		= @iconv_mime_decode($o->to);
			$o->date 	= @iconv_mime_decode($o->date);
			$o->subject = @iconv_mime_decode($o->subject);
		}

		return array_reverse($overviews);
	}
	
	/**
	*	Retrieves the message with the given ID
	*
	*	@param $id (int) The ID of the message
	*	@param $uid (boolean) Whether the given ID is a message ID or
	*		unique ID.
	*
	*	@return (SimpleMailMessage) A message object, false on failure
	*/
	public function getMessage($id, $uid = true, $peek = false)
	{
		try
		{
			return new SimpleMailMessage($this->mbox, $id, $uid, $peek);
		}
		catch(Exception $e)
		{
			return false;
		}
	}
	
	public function getAttachment($uid, $partno, $encoding = 4)
	{
		$data = imap_fetchbody($this->mbox, $uid, $partno, FT_UID);

		if($encoding == 4)
			$data = quoted_printable_decode($data);
		elseif($encoding == 3)
			$data = base64_decode($data);
			
		return $data;
	}
	
	/**
	*	@return (int) Number of unread messages
	*/
	public function unread()
	{
		/*
		$msgs = imap_search($this->mbox, 'UNSEEN');
		return (is_array($msgs)) ? count($msgs) : 0;
		*/
		return isset($this->status->unseen) ? $this->status->unseen : 0;
	}
	
	/**
	*	@return (int) Total number of unread messages
	*/
	public function total()
	{
		return isset($this->status->messages) ? $this->status->messages : 0;
	}
	
	/**
	*	@return (array) An array of mail boxes. Each element contains information for a
	*		particular mailbox.
	*/
	public function getMailBoxes($inc_inbox = false)
	{		
		$boxes = imap_list($this->mbox, $this->mbox_str, '*');
		
		if($boxes)
		{
			$ret = array();
			foreach($boxes as $box)
			{
				$box = str_replace($this->mbox_str, '', $box);

				if($inc_inbox || $box != $this->inbox)
				{
					$nice_name = $this->mailboxNiceName($box);
					$slug = $this->mailboxSlug($box);
					
					$ret[] = (object) array('display_name' => $nice_name, 'slug' => $slug, 'raw' => $box);
				}
			}
			$this->boxes = $ret;
			return $ret;
		}
		return array();
	}
	
	public function getBoxBySlug($slug)
	{
		if(!$this->boxes)
			$this->getMailBoxes(true);
		foreach($this->boxes as $box)
		{
			if($box->slug == $slug)
				return $box->raw;
		}
		
		print_r(debug_backtrace());
	}
	
	/**
	*	Deletes emails with the given IDs
	*
	*	@param $messages (array) An array of UIDs (integer)
	*	@return (bool) true on success, false otherwise
	*/
	public function deleteMessages($messages)
	{
		if(!is_array($messages) || count($messages) < 1)
			return false;
		
		$ret = true;
		foreach($messages as $uid)
			$ret = $ret & imap_delete($this->mbox, trim($uid), FT_UID);
			
		return $ret;
	}
	
	/**
	*	Moves messages with the given IDs to the specified mailbox
	*
	*	@param $messages (array) An array of UIDs
	*	@param $destination (string) The mailbox to move the messages to
	*	@return (bool) true on success, false otherwise
	*/
	public function moveMessages($messages, $destination)
	{
		if(!is_array($messages) || count($messages) < 1)
			return false;
		
		$msg_str = implode(',', $messages);

		return imap_mail_move($this->mbox, $msg_str, $destination, CP_UID);
	}
	
	/**
	*	Sets/unsets a flag for messages with the given IDs
	*
	*	@param $messages (array) An array of UIDs
	*	@param $set (bool) Whether to set or unset the flag
	*	@param $flag (string) the flag to set
	*	@return (bool) true on success, false otherwise
	*/
	public function flagMessages($messages, $set, $flag)
	{
		if(!is_array($messages) || count($messages) < 1)
			return false;
			
		$msg_str = implode(',', $messages);
		
		if($set)
			return imap_setflag_full($this->mbox, $msg_str, $flag, ST_UID);
		
		return imap_clearflag_full($this->mbox, $msg_str, $flag, ST_UID);
	}
	
	/**
	*	Generates a slug from the given mailbox name
	*
	*	@param $box (string) The name of the mailbox
	*	@return (string) The slug
	*/
	public function mailboxSlug($box)
	{
		//return urlencode($box);
		//return sanitize_title($box);
		return strtr(base64_encode($box), '+/=', '-_,');
	}
	
	public function slugToMailbox($slug)
	{
		return base64_decode(strtr($slug, '-_,', '+/='));
	}
	
	/**
	*	Generates a 'nice name' from a mailbox name
	*
	*	@param $box (string) The name of the mailbox
	*	@return (string) The nice name
	*/
	public function mailboxNiceName($box)
	{
		$index = (int) ( (strrpos($box, '.') !== false) ? strrpos($box, '.') : strrpos($box, '/') );
		if($index)
			return ucwords(strtolower(substr($box, $index+1)));
			
		return ucwords(strtolower($box));
		/*
		$regex = '/^' . $this->inbox . '(\.|\/)/';
		return ucwords(strtolower(preg_replace($regex, '', $box, 1)));
		*/
	}
}
?>